#define SAVE_SCREEN


#include "crogue.h"

// Evil, evil, bastardized, disgusting, vomit-inducing hack
// In response to how the linking system produces larger outputs when code is
// split into multiple files. Only use when compiling for a calculator.
#ifdef IS_CALCULATOR

#include "construct.c"
#include "draw.c"
#include "items.c"
#include "magic.c"
#include "mkmap.c"
#include "monst.c"
#include "player.c"
#include "tiles.c"
#include "files.c"
#include "rle.c"
#include "debug.c"
#include "util.c"
#include "spell.c"

#include "tilefield.c"
#include "messagefield.c"
#include "menu.c"
#include "help.c"
#include "prompt.c"
#include "drawstring.c"

#endif // IS_CALCULATOR

static void initialize(void);
static void eventloop(void);

//{{{
//
// Output an error message and quit.
//
void fatal(char *msg)
{
#ifdef UNIX
	FILE *log = fopen("log.txt", "w");
	if(log)
		fprintf(log, "%s", msg);
	endwin();
#endif
	clrscr();
	puts(msg);
	READCHAR;
	quit();	// cleans up memory and exits
}
//}}}

#ifdef IS_CALCULATOR
//{{{
void _main(void)
{
	initialize();
	eventloop();
}
//}}}
#else
//{{{
int main(int argc, char **argv)
{
	initialize();
	eventloop();
	return 0;
}
//}}}
#endif


//{{{
static void initialize(void)
{
	int loadedfromfile = 0;
	int i;
#ifndef IS_CALCULATOR
	FILE *constfile;
#endif
	
	clrscr();
	
	// Claim memory for main data struct
	w = (world*)malloc(sizeof(world));
	if(!w) {
		fatal("Out of memory.");
	}
	
	w->items.items = NULL;
	w->plr.spellknowledge = NULL;
	
	INIT_RNG;		// FIXME: the TI-89 library random number generator has problems. Find a better replacement.
	
	// Load saved game if possible
	loadedfromfile = restore_game();
	
	// Tilefield widget allocates memory, so check for failure
	if( !UI_TF_Init( &w->tilefield,
	                 COORD(0,12),
	                 COORD(MAPSIZE_X,MAPSIZE_Y),
#	                 ifdef REALCOMPUTER
	                	 COORD(64,20),
#	                 endif
#	                 ifdef USE_TI89
	                	 COORD(32,16),
#	                 endif
#	                 ifdef USE_TI92P
		                 COORD(48,20),
#	                 endif
	                 tiles,
	                 5, 5) )
		fatal("Out of memory.");
	
	UI_MF_clear();
	
	// Initialize graphics
#ifdef IS_CALCULATOR // Grayscale
	if(GrayOn()==FALSE)
		fatal("Out of memory.");
	if(AMS_2xx)
		SetAlphaStatus(FALSE);
	FontSetSys(F_4x6);
#endif
#ifdef UNIX // Curses
	initscr();
	noecho();
	cbreak();
	notimeout(stdscr, TRUE);
	keypad(stdscr, TRUE);
#endif
	
	// Load constant-data file
#ifdef IS_CALCULATOR
	HSym datfile;
	datfile = SymFind(DATAFILENAME);
	w->constfile = H_NULL;
	if(datfile.folder==0)
		fatal("Data file not found.");
	w->constfile = DerefSym(datfile)->handle;
	w->constfileoffset = HLock(DerefSym(datfile)->handle) + 2;
	memcpy(&w->desc, w->constfileoffset, sizeof(world_descriptor));
#else
	constfile = fopen("rogdat.dat", "rb");
	if(!constfile)
		fatal("Couldn't open constant data file.");
	w->constfileoffset = malloc(65536);	// Burns 64K, but only on PC
	fread(w->constfileoffset, 65536, 1, constfile);
	memcpy(&w->desc, w->constfileoffset, sizeof(world_descriptor));
	fclose(constfile);
#endif
	w->tiledescs     = (tiledesc* )CONSTFILEDEREF(w->desc.tileoffset);
	w->mapdescs      = (mapdesc*  )CONSTFILEDEREF(w->desc.mapoffset);
	w->itemdescs     = (filelink* )CONSTFILEDEREF(w->desc.itemoffset);
	w->spelldescs    = (spelldesc*)CONSTFILEDEREF(w->desc.spelllist);
	w->playerclasses = (classdesc*)CONSTFILEDEREF(w->desc.playerclasses);
	w->shuffledata   = (filelink* )CONSTFILEDEREF(w->desc.stdata);
	
#ifdef IS_CALCULATOR
	// Display title screen
	GraySetAMSPlane(DARK_PLANE);
	BitmapPut(0, 0, CONSTFILEDEREF(w->desc.titlescreen_dark_plane), &fullScreen, A_REPLACE);
	GraySetAMSPlane(LIGHT_PLANE);
	BitmapPut(0, 0, CONSTFILEDEREF(w->desc.titlescreen_light_plane), &fullScreen, A_REPLACE);
#endif
	
	// If not loaded from saved game, initialize things
	if(!loadedfromfile) {
		init_shuffle_tab();
		
		w->level=w->maxlevel=1;
		w->messagevis=0;
		w->time=0;
		
		// Allocate memory for player spell knowledge
		init_spells();
		
		// Allocate memory for item identification info
		w->itemids = (ulong*)malloc( 32 );
		if(!w->itemids)
			fatal("Out of memory.");
		for(i=0; i < w->desc.itementries/32 + 1; i++)
			w->itemids[i] = 0;
		for(i=0; i<w->desc.itementries; i++) {
			if( ITEMDESCN(i).flags & ITEMFLAG_AUTOID )
				identify_type(i);
		}
		
		init_player();
		gen_map(0, COORD(0,0));
	}
	
	draw_all_tiles();
	draw();
	enter_map();
}
//}}}

//{{{
static void eventloop(void)
{
	while(1)
	{
		draw();
		plr_move();
	}
}
//}}}

//{{{
void die(int m)
{
	if(m)
		message("You die...");
	
	message("Your score was %li.", w->plr.score);
	
	draw();
	while(READCHAR != KEY_ACKNOWLEDGE);
	
	cleanup_save();
	quit();
}
//}}}


//{{{
void quit(void)
{
#ifdef IS_CALCULATOR
	if(w->constfile != H_NULL)
		HeapUnlock(w->constfile);
	
	GrayOff();
#else
	free(w->constfileoffset);
#endif
	
#ifdef UNIX
	endwin();
#endif
	
	if(w->plr.spellknowledge)
		free(w->plr.spellknowledge);
	
	cleanup_items();
	UI_TF_Deinit(&w->tilefield);
	if(w->itemids) free(w->itemids);
	if(w) free(w);
	
	exit(0);
}
//}}}
//{{{
void count_timer(int amt, uint *timer)
{
	if(amt>=*timer)
		*timer = 0;
	else
		*timer -= amt;
}
//}}}
//{{{
void timepass(int amt)
{
	uint partialhpsperhp;
	uint partialppsperpp;
	uint i;
	
	// Regenerate hit points
	partialhpsperhp = 200+10*w->plr.level;
	w->plr.partialhps += w->plr.hps_max * amt;
	w->plr.hps += w->plr.partialhps / partialhpsperhp;
	w->plr.hps += w->plr.extrinsic[STAT_REGEN];
	w->plr.partialhps -= (w->plr.partialhps/partialhpsperhp)*partialhpsperhp;
	if( w->plr.hps > w->plr.hps_max )
		w->plr.hps = w->plr.hps_max;
	
	// Regenerate power points
	partialppsperpp = 200+20*w->plr.level;
	w->plr.partialpps += (w->plr.pps_max + w->plr.extrinsic[STAT_MANA]) * amt;
	w->plr.pps += w->plr.partialpps / partialppsperpp;
	w->plr.partialpps -= (w->plr.partialpps/partialppsperpp)*partialppsperpp;
	if( w->plr.pps > w->plr.pps_max )
		w->plr.pps = w->plr.pps_max;
	
	// Count down affliction timers
	count_timer(amt, &w->plr.hallutime);
	count_timer(amt, &w->plr.conftime);
	count_timer(amt, &w->plr.paraltime);
	
	w->plr.satiation -= amt;
	if(w->plr.satiation <= -50) {
		message("You are starving!");
		plr_takedamage(-w->plr.satiation/50);	// FIXME: PV shouldn't affect this
												// FIXME: should scale with number of turns spend
	}
	
	// Run monster AI
	for(i=0; i<MONSTERS_MAX; i++)
	{
		if(!isNull(w->m[i].type))
		{
			w->m[i].energy += MDESC(i)->speed * amt;
			
			while(w->m[i].energy >= w->plr.extrinsic[STAT_SPEED])
			{
				monstmove(i);
				w->m[i].energy -= w->plr.extrinsic[STAT_SPEED];
			}
		}
	}
	
	// Add monsters randomly with time
	// (1 in 40 chance per turn)
	for(i=0; i<amt; i++)
		if( !(w->mapdescs[w->level].flags & MAPFLAG_NORANDMONST) && rand()%40 == 0 )
			addrandommonst();
	
	w->time++;
}
//}}}

