#include "crogue.h"

uint is_in_vision_range(int x, int y);

//{{{
uint is_in_vision_range(int x, int y)
{
	register int distance = distancesquare(x, y, w->plr.x, w->plr.y);
	
	if(distance <= 2 || distance <= w->plr.extrinsic[STAT_LIGHTRADIUS])
		return 1;
	else
		return 0;
}
//}}}
//{{{
void calc_light(void)
{
	int yi, xi;
	
	for(yi=0; yi<MAPSIZE_Y; yi++)
		for(xi=0; xi<MAPSIZE_X; xi++)
		{
			// Find unlit tiles to light
			if(!(w->t[yi][xi].flags & TFLAG_LIT))
			{
				if( (
						is_in_vision_range(xi, yi) ||
				        w->t[yi][xi].flags & TFLAG_INTRINSIC_LIGHT
				    ) &&
					tracevision(xi, yi, w->plr.x, w->plr.y) )
				{
					if( w->t[yi][xi].flags & TFLAG_INTRINSIC_LIGHT && !TILEDESC(w->t[yi][xi]).transparent &&
					    !is_in_vision_range(xi, yi) )
					{
						int xd, yd;
						
						xd = xi;
						yd = yi;
						
						if(w->plr.x > xi)
							xd++;
						else if(w->plr.x < xi)
							xd--;
						
						if(w->plr.y > yi)
							yd++;
						else if(w->plr.y < yi)
							yd--;
						
						if(!(w->t[yd][xd].flags & TFLAG_INTRINSIC_LIGHT))
							continue;
					}
					w->t[yi][xi].flags |= TFLAG_LIT;
					w->t[yi][xi].flags |= TFLAG_EXPLORED;
					draw_tile(xi, yi);
				}
			}
			// Find lit tiles to unlight
			else if(w->t[yi][xi].flags & TFLAG_LIT)
			{
				if( (
						!is_in_vision_range(xi, yi) &&
				        !(w->t[yi][xi].flags & TFLAG_INTRINSIC_LIGHT)
				    ) ||
					!tracevision(xi, yi, w->plr.x, w->plr.y) )
				{
					w->t[yi][xi].flags &= ~TFLAG_LIT;
					draw_tile(xi, yi);
				}
			}

		}
}
//}}}
//{{{
char* hunger_status(void)
{
	if(w->plr.satiation <= -50)
		return " starving";
	else if(w->plr.satiation <= 50)
		return " weak";
	else if(w->plr.satiation < 200)
		return " hungry";
	else
		return "";
}
//}}}


#ifdef REALCOMPUTER
coord statuslinepos = {0, 22};
#endif


//{{{
void draw(void)
{
	char outbuf[64];
	
	if(w->messagevis)
	{
		if(!w->plr.paraltime)
			w->messagevis = 0;
	}
	else
		UI_MF_clear();
	
	// Draw statusbar
	sprintf(outbuf, "HP%i/%i PW%i/%i LV%i %s%s%s%s",
		w->plr.hps,
		w->plr.hps_max,
		w->plr.pps,
		w->plr.pps_max,
		w->plr.level,
		(const char*)CONSTFILEDEREF(w->mapdescs[w->level].name),
		hunger_status(),
		(w->plr.conftime)?" conf":"",
		(w->plr.hallutime)?" hallu":""
	);

#ifdef USE_TI89
	SCR_RECT rect;
	rect = (SCR_RECT){{0, 92, 159, 99}};
	Black_ScrRectFill(&rect, &rect, A_REVERSE);
	Black_DrawStr(0, 94, outbuf, A_NORMAL);
#endif
#ifdef USE_TI92P
	SCR_RECT rect;
	rect = (SCR_RECT){{0, 120, 239, 127}};
	Black_ScrRectFill(&rect, &rect, A_REVERSE);
	Black_DrawStr(0, 120, outbuf, A_NORMAL);
#endif
#ifdef REALCOMPUTER
	clear_line(statuslinepos.y);
	clear_line(statuslinepos.y+1);
	gotoxy(statuslinepos.x, statuslinepos.y);
	printw("%s", outbuf);
#endif
}
//}}}
//{{{
void draw_all_tiles(void)
{
	int x, y;
	
	for(y=0; y<MAPSIZE_Y; y++)
		for(x=0; x<MAPSIZE_X; x++)
			draw_tile(x, y);	
}
//}}}

//{{{
void draw_tile(ushort x, ushort y)
{
	uchar tilenum=1;
	int filled;
	int lit;
	uchar lighting=LIGHTING_DARK;
	
	if( !(w->t[y][x].flags & TFLAG_EXPLORED) ) {
		tilenum = 1;
		lighting = LIGHTING_UNEXPLORED;
		goto done;
	}
	
	filled = w->t[y][x].flags & TFLAG_OCCUPIED;
	
	lit = w->t[y][x].flags & TFLAG_LIT;
	
	if(lit)
		lighting = LIGHTING_LIT;
	
	if( filled && lit ) {
		if(x==w->plr.x && y==w->plr.y)
			tilenum = '@';
		else
			if( w->plr.hallutime ) {
				tilenum = RANGE('Z', 'A');
			} else {
				tilenum = MDESC( monstbytile(x, y) )->drawchar;
			}
	} else if(w->t[y][x].numitems) {
		tilenum = draw_item(x, y);
	} else {
		if(w->t[y][x].flags & TFLAG_HIDETRAP)
			tilenum = w->tiledescs[TILE_FLOOR].drawchar;
		else
			tilenum = w->tiledescs[w->t[y][x].type].drawchar;
	}
	
done:
	UI_TF_PutTile(&w->tilefield, x, y, tilenum, lighting);
}
//}}}

