//{{{
typedef struct weapon_info	// 2 bytes
{
	schar plus;
	uchar special;
} weapon_info;

typedef struct ammo_info	// 2 bytes
{
	schar plus_hit;
	schar plus_dam;
} ammo_info;

typedef struct armor_info	// 2 bytes
{
	schar plus;
	uchar special;
} armor_info;

typedef struct magic_info	// 1 byte
{
	uchar nothing;
} magic_info;

typedef struct spellbook_info
{
	schar readings;
} spellbook_info;

typedef union iteminfo	// 4 bytes
{
	weapon_info    weap;
	armor_info     armr;
	magic_info     mag;
	ammo_info      ammo;
	spellbook_info book;
} iteminfo;
//}}}


typedef struct item	// 12 bytes
{
	ushort type;
	ushort stacksize;
	iteminfo info;
	ushort next;
	uchar flags;
	uchar pad2;
} item;

#define ITEMFLAG_EQUIPPED   1
#define ITEMFLAG_CURSED     2
#define ITEMFLAG_IDENTIFIED 4


typedef struct itemsinfo
{
	uint num;
	uint alloced;
	item *items;
} itemsinfo;

/*{{{*/
enum {
	type_misc=0,
	type_weapon=1,
	type_armor=2,
	type_magic=3,
	type_ammo=4,
	type_spellbook=5
};

typedef struct misc_desc
{
	uchar nothing;
} misc_desc;

typedef struct weapon_desc
{
	range damage;
	uchar speed;
	schar to_hit;
	
	range missiledamage;
	uchar missilerange;
	
	uchar launcher;
} weapon_desc;

typedef struct stat_mod_desc
{
	uchar type;
	uchar stat;
	schar amount;
} stat_mod_desc;
#define STATMOD_ADD     1
#define STATMOD_ADDPLUS 2 // Same as add, but if the item is +N, add N to amount
#define STATMOD_OR      3

typedef struct armor_desc
{
	uchar num_effects;
	stat_mod_desc effects[1];
} armor_desc;

typedef struct magic_desc
{
	uchar flags;
} magic_desc;
#define MAGFLAG_DISAPPEAR 1
#define MAGFLAG_SEPARATE 2

typedef struct ammo_desc
{
	range damage;
	schar to_hit;
	uchar distance;
	uchar launch_from;
} ammo_desc;

typedef struct spellbook_desc
{
	uint spellid;
} spellbook_desc;

typedef union item_details_desc
{
	misc_desc misc;
	weapon_desc weapon;
	armor_desc armor;
	magic_desc magic;
	ammo_desc ammo;
	spellbook_desc spellbook;
} item_details_desc;
/*}}}*/
//{{{
typedef struct itemdesc
{
	uint num;
	filelink name;
	filelink unidname;
	uchar drawchar;
	uchar type;
	schar spot;
	uchar category;
	uint flags;
	uint weight;
	uint stacksize;
	item_details_desc desc;
} itemdesc;

#define ITEMFLAG_STACK     1
#define ITEMFLAG_FRAGILE   2
#define ITEMFLAG_AUTOID    4
#define ITEMFLAG_RUSTPROOF 8
#define ITEMFLAG_LAUNCHER  16
#define ITEMFLAG_CHARGED   32
//}}}


typedef int (*inventoryfilter)(item*);

// Inventory filters
int filter_none(item *i);
int filter_cursed(item *i);
int filter_worn(item *i);
int filter_wearable(item *i);
int filter_unidentified(item *i);
int filter_usable(item *i);

// Inventory menus
int pick_item(inventoryfilter f);
int filter_matches(inventoryfilter f);

// Player inventory
void init_inventory(void);
item split_item(int slot);
void merge_item(int slot, item it);
int give_item(const item* item, int silent);

// Equipment handling
item *player_equipment(int slot);

// Utility functions
const char *shortitemname(const item *i);
uchar *itemname(const item *i);

// Misc
void list_discoveries(void);
void identify_type(int t);
ulong type_is_identified(ulong t);

// Item storage
void compress_items(void);
void place_item(item i, uchar x, uchar y);
uchar draw_item(uchar x, uchar y);
void init_items();
void cleanup_items(void);
uint add_item(item itm);
void delete_item(uint which);

item randitem(void);

// Player actions
void drop_item(void);
void wear_item(void);
void takeoff_item(void);
void pickup(void);


