
#ifndef TARGET
#  error No target specified
#endif

#define T_TI89  1
#define T_TI92P 2
#define T_WIN32 3
#define T_UNIX  4

//{{{
#if TARGET == T_TI89
#	define IS_CALCULATOR
#	define USE_TIGCCLIB
#	define USE_TI89
#endif
#if TARGET == T_TI92P
#	define IS_CALCULATOR
#	define USE_TIGCCLIB
#	define USE_TI92P
#endif
#if TARGET == T_WIN32_MSC
#	define REALCOMPUTER
#	define WIN32
#	define WINDOWS
#	define WIN32_MSC
#endif
#if TARGET == T_WIN32_MINGW
#	define REALCOMPUTER
#	define WINDOWS
#	define WIN32_MINGW
#endif
#if TARGET == T_UNIX
#	define REALCOMPUTER
#	define UNIX
#endif

#ifdef REALCOMPUTER
#	undef REGISTERVAR // Can't specify m68k registers on a PC
#endif
//}}}

//{{{
// Libraries
#ifdef USE_TIGCCLIB
#	include <tigcclib.h>
#	define INIT_RNG randomize()
#	define READCHAR ngetchx()
	extern const SCR_RECT fullScreen;

//	HACK: This needs to be defined even on AMSes that don't support it
#	define SetAlphaStatus _rom_call(void,(char),163)
#endif

#ifdef REALCOMPUTER
#	include <stdio.h>
#	include <stdlib.h>
#	include <time.h>
#	include <stdarg.h>
#	include <ctype.h>
#	include <string.h>
#	define INIT_RNG srand(time(NULL))
#endif

#ifdef WIN32_MSC
#	define WIN32_LEAN_AND_MEAN
#	include <windows.h>
#	include <wincon.h>
#	define READCHAR getch()
#endif

#ifdef WIN32_MINGW
#	define WIN32_LEAN_AND_MEAN
#	include <windows.h>
#	include <conio.h>
#	define READCHAR getch()
#endif

#ifdef UNIX
#	include <ncurses.h>
#	define READCHAR (refresh(), wgetch(stdscr))
#endif
//}}}

//{{{
// Typedefs for brevity (and for length compatibility between 16- and 32-bit
// int sizes).
#define uchar  unsigned char
#define schar  signed char
#define ushort unsigned short
#define sshort signed short
#define uint   unsigned short
#define sint   signed short
#define ulong  unsigned long
#define slong  signed long
//}}}

//{{{
// GCC extensions
#ifdef __GNUC__
#	define ATTRIB(a) __attribute((a))
#	define CASTCONSTRUCT
#else
#	define ATTRIB(a)
#endif

// MSC BS workarounds
#ifdef _MSC_VER
#	pragma warning ( disable : 4761 ) // Suppress integral size mismatch warnings
#endif

// Cast constructors workaround
#ifdef CASTCONSTRUCT
#	define COORD(a,b) ((coord){(a),(b)})
#	define WINCOORD(a,b) ((COORD){(a),(b)})
#	define SRANGE(a,b) ((range){(a),(b)})
#	define DIRECTION(a,b) ((direction){(a),(b)})
#else
#	define COORD(a,b) construct_coord(a, b)
#	define WINCOORD(a,b) construct_wincoord(a, b)
#	define SRANGE(a,b) construct_range((a),(b))
#	define DIRECTION(a,b) (construct_direction((a),(b)))
#endif
//}}}

//{{{
// Stuff in TIGCCLIB which we replace for PC
#ifndef USE_TIGCCLIB
#	define F_4x6 0
#	define FontSetSys(a)

#	ifdef WIN32
		extern int scratch;
#		define gotoxy(x,y) (SetConsoleCursorPosition( GetStdHandle(STD_OUTPUT_HANDLE), WINCOORD((x), (y)) ))
#		define clrscr() (FillConsoleOutputCharacter( GetStdHandle(STD_OUTPUT_HANDLE), ' ', 80*25, WINCOORD(0,0), &scratch ), gotoxy(0,0))
#		define clrtiles() (FillConsoleOutputCharacter( GetStdHandle(STD_OUTPUT_HANDLE), ' ', 80*20, WINCOORD(0,2), &scratch ))
#		define clear_line(a) (FillConsoleOutputCharacter( GetStdHandle(STD_OUTPUT_HANDLE), ' ', 80, WINCOORD(0, a), &scratch ))
#		define printw printf
#		define addch putchar
#		define A_NORMAL 0
#	endif
#	ifdef UNIX
#		define clear_line(a) mvprintw(a, 0, "                                                                                ");
#		define gotoxy(a,b) move(b,a)
#		define clrscr() clear()
#		define clrtiles() erase()
#	endif
#endif
//}}}

//{{{
#ifdef REALCOMPUTER
#	define SCREEN_COLUMNS 80
#	define SCREEN_ROWS    25
#else
#	define SCREEN_COLUMNS 160
#	define SCREEN_ROWS    (100/6)
#endif
//}}}

typedef struct coord
{
	uchar x;
	uchar y;
} coord;

typedef struct direction
{
	schar x, y;
} direction;


typedef struct range
{
	uchar max;
	uchar min;
} range;


typedef struct lrange
{
	sint max;
	sint min;
} lrange;


// Construct.c
#ifndef CASTCONSTRUCT
	coord construct_coord(uchar x, uchar y);
#	ifndef WINDOWS
		typedef struct coord COORD;
#	endif
	COORD construct_wincoord(int x, int y);
	direction construct_direction(schar x, schar y);
	range construct_range(uchar max, uchar min);
#endif


