#include "crogue.h"

static void throw_item(int which_item);
static void thrown_item_effect(int type, int monst);

void detect_objects(void);
void dig_ray(int x, int y, direction dir, int distance);
void removecurses(void);
int secretdoordetect(void);

//{{{
void throw_item_from_menu(void)
{
	int which;
	
	if(filter_matches(filter_none)==0)
	{
		message("You don't have anything to throw.");
		return;
	}
	message("Throw what?");
	which = pick_item(filter_none);
	
	if(which<0)
	{
		message("Never mind.");
		return;
	}
	
	throw_item(which);
}
//}}}
//{{{
void throw_readied_item(void)
{
	// FIXME: This should be handled through player_equipment()
	int i;
	for(i=0; i<PLRINVSIZE; i++)
		if(INVENTORY(i).type && ITEMDESC(INVENTORY(i)).spot == 1 && INVENTORY(i).flags & ITEMFLAG_EQUIPPED)
		{
			throw_item(i);
			return;
		}
	
	message("You don't have any ammunition readied.");
}
//}}}


//{{{
void throw_item(int which_item)
{
	direction dir;
	coord landing;
	int monst;
	int finishlanding = 1;
	item projectile;
	int tohitmod = 0;
	int range = 5;
	int damage = 0;
	
	projectile = split_item(which_item);
	
	switch( ITEMDESC(projectile).type ) {
		case type_armor:
			if( projectile.flags & ITEMFLAG_EQUIPPED ) {
				message("You can't throw something you're wearing.");
				goto abort;
			}
			damage = 1;
			tohitmod = -3;
			break;
		case type_weapon:
			damage += rrandom(ITEMDESC(projectile).desc.weapon.damage);
			damage += projectile.info.weap.plus;
			tohitmod = -3;
			if(projectile.flags & ITEMFLAG_EQUIPPED)
				update_player();
			break;
		case type_ammo:
			if( player_equipment(0)==NULL || !(ITEMDESC(*player_equipment(0)).flags & ITEMFLAG_LAUNCHER) ) {
				message("You need a launcher for that ammo.");
				goto abort;
			}
			if( ITEMDESC(*player_equipment(0)).desc.weapon.launcher != ITEMDESC(projectile).desc.ammo.launch_from ) {
				message("You need an appropriate launcher for that ammo.");
				goto abort;
			}
			else {
				range = ITEMDESC(projectile).desc.ammo.distance;
				
				tohitmod += ITEMDESC(projectile).desc.ammo.to_hit;
				tohitmod += projectile.info.ammo.plus_hit;
				
				damage = rrandom(ITEMDESC(projectile).desc.ammo.damage);
				damage += projectile.info.ammo.plus_dam;
			}
			break;
		default:
			damage = 1;
			break;
	}
	
	projectile.flags &= ~ITEMFLAG_EQUIPPED; // Unmark ammo as 'readied'
	
	dir = prompt_dir("In what direction");
	
	landing = trace_path(w->plr.x, w->plr.y, dir, range);
	
	// If a wall was hit, back up one tile to find the contact point
	if(!w->tiledescs[w->t[landing.y][landing.x].type].passable) {
		landing.x -= dir.x;
		landing.y -= dir.y;
	}
	
	// If a monster was hit, hurt it a little and (maybe) do magical effects
	monst = monstbytile(landing.x, landing.y);
	if(monst != -1) {
		if(plr_attack_monster_roll(monst, tohitmod)) {
			message("The %s hits the %s.", itemname(&projectile), monstname(monst));
			
			switch( ITEMDESC(projectile).type ) {
				case type_magic:
					finishlanding = 0;
					thrown_item_effect(projectile.type, monst);
					break;
			}
			monst_takedamage(monst, damage, 0);
		} else {
			message("The %s misses the %s.", itemname(&projectile), monstname(monst));
		}
	}
	
	if( finishlanding ) {
		if( ITEMDESC(projectile).flags & ITEMFLAG_FRAGILE ) {
			message("The %s shatters!", shortitemname(&projectile));
		} else if(RANGE(3,1) == 1) {
			message("The %s breaks.", shortitemname(&projectile));
		} else {
			place_item(projectile, landing.x, landing.y);
			draw_tile(landing.x, landing.y);
		}
	}
	
	timepass(1);
	return;

abort:
	merge_item(which_item, projectile);
}
//}}}
//{{{
void thrown_item_effect(int type, int monst)
{
	switch(type) {
		case ITYPE_POTION_HEAL:
			message("The %s looks better.", monstname(monst)), 
			monst_heal(monst, RANGE(20,10));
			break;
		case ITYPE_POTION_EXTRAHEAL:
			message("The %s looks much better.", monstname(monst));
			monst_heal(monst, RANGE(70,35));
			break;
		case ITYPE_POTION_FULLHEAL:
			message("The %s looks completely healed.", monstname(monst));
			monst_heal(monst, 1000);
			break;
		case ITYPE_POTION_SICKNESS:
			message("The %s looks weaker.", monstname(monst));
			monst_takedamage(monst, 12, 0);
			break;
		default:
			break;
	}
}
//}}}
//{{{
void use_item(void)
{
	int which;
	int identify_it = 0;
	int itemtype;
	
	if( filter_matches(filter_usable) == 0 )
	{
		message("You don't have anything you can use.");
		return;
	}
	message("Use what?");
	which = pick_item(filter_usable);
	
	if(which<0) {
		message("Never mind.");
		return;
	}
	
	if(ITEMDESC(INVENTORY(which)).type == type_spellbook)
	{
		read_spellbook(which);
		return;
	}
	
	if(ITEMDESC(INVENTORY(which)).type != type_magic)
	{
		message("You can't use that.");
		return;
	}
	
	itemtype  = INVENTORY(which).type;
	
	if(INVENTORY(which).stacksize > 0) {
		switch( itemtype )
		{
			/////////////////////////////////////////////////////////////////////////////////////
			case ITYPE_WAND_NOTHING:
				message("Nothing happens.");
				break;
			
			case ITYPE_WAND_STRIKING:
				if( bolt( w->plr.x, w->plr.y, prompt_dir("In what direction"), 6 ) )
					identify_it = 1;
				break;
			
			case ITYPE_WAND_DIGGING:
				identify_it = 1;
				dig_ray(w->plr.x, w->plr.y, prompt_dir("In what direction"), RANGE(8, 4));
				break;
			
			case ITYPE_WAND_LIGHT:
				identify_it = 1;
				message("There is a sudden flash of light!");
				illuminate(w->plr.x, w->plr.y, 36);
				calc_light();
				break;
			
			case ITYPE_WAND_DETECT:
				identify_it = secretdoordetect();
				break;
			/////////////////////////////////////////////////////////////////////////////////////
			case ITYPE_POTION_HEAL:
				message("You feel better.");
				plr_heal(RANGE(20,10));
				identify_it=1;
				break;
			case ITYPE_POTION_EXTRAHEAL:
				message("You feel much better.");
				plr_heal(RANGE(70,35));
				identify_it=1;
				break;
			case ITYPE_POTION_FULLHEAL:
				message("You feel much better.");
				plr_heal(10000);
				identify_it=1;
				break;
			
			case ITYPE_POTION_SICKNESS:
				message("This stuff tastes like poison.");
				plr_mod_stat(0, -RANGE(5,1));
				identify_it=1;
				break;
			case ITYPE_POTION_HALLU:
				message("Oh wow! Everything looks cosmic now!");
				w->plr.hallutime = RANGE(150, 100);
				identify_it=1;
				break;
			case ITYPE_POTION_CONF:
				message("Huh, What? Where am I?");
				w->plr.conftime = RANGE(40, 15);
				identify_it=1;
				break;
			
			case ITYPE_POTION_OBJDETECT:
				detect_objects();
				identify_it=1;
				break;
			
			case ITYPE_POTION_GAINST:
				plr_mod_stat(0, RANGE(2,1));
				identify_it=1;
				break;
			case ITYPE_POTION_GAINDX:
				plr_mod_stat(1, RANGE(2,1));
				identify_it=1;
				break;
			case ITYPE_POTION_GAINTO:
				plr_mod_stat(2, RANGE(2,1));
				identify_it=1;
				break;
			case ITYPE_POTION_GAINMN:
				plr_mod_stat(3, RANGE(2,1));
				w->plr.pps = w->plr.pps_max;
				identify_it=1;
				break;
			case ITYPE_POTION_GAINWI:
				plr_mod_stat(4, RANGE(2,1));
				identify_it=1;
				break;
			
			case ITYPE_SCROLL_BLANK:
				message("This scroll is completely blank.");
				identify_it=1;
				goto skip_chargedrain;
			case ITYPE_SCROLL_MAGICMAP:
				magic_map();
				identify_it=1;
				break;
			case ITYPE_SCROLL_TELEPORT:
				teleport_player();
				identify_it=1;
				break;
			case ITYPE_SCROLL_IDENTIFY:
				message("This is a scroll of identify.");
				identify();
				identify_it=1;
				break;
			case ITYPE_SCROLL_UNCURSE:
				message("You feel like someone is helping you.");
				removecurses();
				identify_it=1;
				break;
			
			/////////////////////////////////////////////////////////////////////////////////////
			case ITYPE_FOOD_RATION:
				if(RANGE(1,0)) {
					message("Tastes good.");
					satiate(700);
				} else {
					message("Blech! Rotten food!");
					satiate(600);
				}
				break;
		}
		INVENTORY(which).stacksize--;
		if( INVENTORY(which).stacksize==0 && ITEMDESC(INVENTORY(which)).desc.magic.flags & MAGFLAG_DISAPPEAR ) {
			INVENTORY(which).type = 0;
			w->plr.numitems--;
		}
skip_chargedrain:
		;
	} else {
		message("Nothing happens.");
	}
	
	if(identify_it) {
		identify_type(itemtype);
	}
	
	timepass(1);
}
//}}}


//{{{
void detect_objects(void)
{
	int i, ii;
	
	for(i=0; i<MAPSIZE_Y; i++)
		for(ii=0; ii<MAPSIZE_X; ii++)
			if( w->t[i][ii].numitems ) {
				w->t[i][ii].flags |= TFLAG_EXPLORED;
				draw_tile(ii, i);
			}
	
	message("You sense the presence of objects.");
}
//}}}
//{{{
void magic_map(void)
{
	int i, ii;
	
	for(i=0; i<MAPSIZE_Y; i++)
		for(ii=0; ii<MAPSIZE_X; ii++) {
			w->t[i][ii].flags |= TFLAG_EXPLORED;
			draw_tile(ii, i);
		}
	
	message("A map forms in your mind.");
}
//}}}
//{{{
int bolt(int x, int y, direction dir, int distance)
{
	int monst;
	
	while(distance)
	{
		x += dir.x;
		y += dir.y;
		monst = monstbytile(x, y);
		if( monst != -1 ) {
			message("The bolt hits it!");
			monst_takedamage(monst, RANGE(14,7), 0);
			return 1;
		}
		if( !TILEDESC(w->t[y][x]).transparent )
			return 0;
		distance--;
	}
	return 0;
}
//}}}
//{{{
void dig_ray(int x, int y, direction dir, int distance)
{
	while(distance)
	{
		x += dir.x;
		y += dir.y;
		
		// Don't go off the edge of the screen)
		if(x==0 || y==0 || x+1>=MAPSIZE_X || y+1>=MAPSIZE_Y)
			break;;
		
		switch(w->t[y][x].type)
		{
			case TILE_CDOOR:
			case TILE_LDOOR:
 				message("The door is razed!");
			case TILE_WALL:
				w->t[y][x].type = TILE_FLOOR;
				break;
		}
		
		distance--;
	}
	 
	calc_light();
}
//}}}
//{{{
void identify(void)
{
	int which;
	
	if(filter_matches(filter_unidentified)==0)
	{
		message("You have nothing to identify.");
		return;
	}
	
	message("Identify what?");
	which = pick_item(filter_unidentified);
	
	if(which<0)
	{
		message("You don't identify anything.");
		return;
	}
	
	identify_type( INVENTORY(which).type );
	INVENTORY(which).flags |= ITEMFLAG_IDENTIFIED;
	
	message(itemname(&INVENTORY(which)));
}
//}}}
//{{{
void removecurses(void)
{
	int which;
	
	if(filter_matches(filter_cursed)==0)
	{
		message("You have nothing to uncurse.");
		return;
	}
	message("Uncurse which item?");
	which = pick_item(filter_cursed);
	
	if(which<0)
	{
		message("You waste the scroll.");
		return;
	}
	
	if(INVENTORY(which).flags & ITEMFLAG_CURSED)
	{
		message("It glows for a moment.");
		INVENTORY(which).flags &= ~ITEMFLAG_CURSED;
	} else {
		message("Nothing happens.");
	}
}
//}}}
//{{{
int secretdoordetect(void)
{
	int x, y, ret=0;
	
	for(y=0; y<MAPSIZE_Y; y++)
		for(x=0; x<MAPSIZE_X; x++)
		{
			if(!(w->t[y][x].flags & TFLAG_LIT))
				continue;

			if(w->t[y][x].type == TILE_SDOOR)
			{
				message("You spot a hidden door!");
				w->t[y][x].type = TILE_CDOOR;
				draw_tile(x, y);
				ret=1;
			}
			if(w->t[y][x].flags & TFLAG_HIDETRAP)
			{
				message("You notice a trap!");
				w->t[y][x].flags &= ~TFLAG_HIDETRAP;
				draw_tile(x, y);
				ret=1;
			}
		}
	
	return ret;
}
//}}}

//{{{
void teleport_player(void)
{
	message("Suddenly you stand somewhere else.");
	w->plr.conftime += 5;
	place_player_randomly();
}
//}}}
