#include "crogue.h"

sint mod_from_attribute(uint attr, sint mindistance, sint divisor);

void follow_connection(void);
void plr_walk(ushort x, ushort y);
void plr_moveto(ushort x, ushort y);
void plr_attack( int monstid );
void char_info(void);
void plr_search(void);

//{{{
int plr_attack_monster_roll(int monstid, int modifier)
{
	int roll = RANGE(20,1);
	
	if(roll>=19) return 1;
	if(roll==1)  return 0;
	
	roll += modifier + w->plr.extrinsic[STAT_TOHIT] + 10;
	roll -= RANGE(20,1);
	if(roll >= MDESC(monstid)->dv)
		return 1;
	else
		return 0;
}
//}}}

//{{{
static void playerclassname(int n, char *buf)
{
	strcpy( buf, (char*)CONSTFILEDEREF(w->playerclasses[n].name) );
}
//}}}
//{{{
sint mod_from_attribute(uint attr, sint mindistance, sint divisor)
{
	sint v = w->plr.extrinsic[attr];
	
	v -= 10;
	
	if(v>0)
	{
		v -= mindistance;
		if(v<0)
			v=0;
	}
	if(v<0)
	{
		v += mindistance;
		if(v>0)
			v=0;
	}
	
	v /= divisor;
	
	return v;
}
//}}}
//{{{
void init_player(void)
{
	classdesc *plr_class;
	int i;
	
	// Pick player class
retry:
	w->plr.class = UI_Menu_Pick( COORD(3,60), COORD(45, 35), w->desc.numplayerclasses, &playerclassname, 0 );
	if(w->plr.class==-1) goto retry;
	
	plr_class = &w->playerclasses[w->plr.class];
	
	init_inventory();
	
	// Set up statistics and timers
	for(i=0; i<NUM_STATS; i++)
		w->plr.intrinsic[i] = 0;
	
	w->plr.intrinsic[STAT_STRENGTH]  = rrandom(plr_class->strength  );
	w->plr.intrinsic[STAT_DEXTERITY] = rrandom(plr_class->dexterity );
	w->plr.intrinsic[STAT_TOUGHNESS] = rrandom(plr_class->toughness );
	w->plr.intrinsic[STAT_MANA]      = rrandom(plr_class->mana      );
	w->plr.intrinsic[STAT_WILLPOWER] = rrandom(plr_class->willpower );
	
	w->plr.intrinsic[STAT_LIGHTRADIUS] = 1;
	w->plr.intrinsic[STAT_SPEED] = 10;
	
	w->plr.hps = w->plr.hps_max = w->plr.intrinsic[STAT_TOUGHNESS];
	w->plr.pps = w->plr.pps_max = w->plr.intrinsic[STAT_MANA];
	w->plr.partialhps = w->plr.partialpps = 0;
	w->plr.level = 1;
	w->plr.xp = 0;
	w->plr.xpdiffactor = -5;
	w->plr.satiation = 1000;
	
	w->plr.conftime = 0;
	w->plr.hallutime = 0;
	w->plr.paraltime = 0;
	
	w->plr.score = 0;
	
	update_player();
}
//}}}
//{{{
void update_player(void)
{
	int i, j;
	item* item;
	
	// Derived intrinsics
	w->plr.intrinsic[STAT_TOHIT]       = w->plr.intrinsic[STAT_STRENGTH]/6 + w->plr.level;
	w->plr.intrinsic[STAT_DAMAGE_MAX]  = w->plr.intrinsic[STAT_STRENGTH]/4;
	w->plr.intrinsic[STAT_DAMAGE_MIN]  = 0;
	
	for(i=0; i<NUM_STATS; i++)
		w->plr.extrinsic[i] = w->plr.intrinsic[i];
	
	for(i=0; i<PLRINVSIZE; i++)
	{
		item = &INVENTORY(i);
		if(!item->type) continue;
		if(!(item->flags & ITEMFLAG_EQUIPPED)) continue;
		
		switch( ITEMDESC(*item).type )
		{
			case type_weapon:
				w->plr.extrinsic[STAT_DAMAGE_MIN] += ITEMDESC(*item).desc.weapon.damage.min + item->info.weap.plus;
				w->plr.extrinsic[STAT_DAMAGE_MAX] += ITEMDESC(*item).desc.weapon.damage.max + item->info.weap.plus;
				w->plr.extrinsic[STAT_TOHIT]      += ITEMDESC(*item).desc.weapon.to_hit + item->info.weap.plus;
				break;
				
			case type_armor:
				for(j=0; j<ITEMDESC(*item).desc.armor.num_effects; j++)
				{
					switch(ITEMDESC(*item).desc.armor.effects[j].type)
					{
						case STATMOD_ADD:
							w->plr.extrinsic[ ITEMDESC(*item).desc.armor.effects[j].stat ] += ITEMDESC(*item).desc.armor.effects[j].amount;
							break;
						case STATMOD_ADDPLUS:
							w->plr.extrinsic[ ITEMDESC(*item).desc.armor.effects[j].stat ] += ITEMDESC(*item).desc.armor.effects[j].amount;
							w->plr.extrinsic[ ITEMDESC(*item).desc.armor.effects[j].stat ] += item->info.armr.plus;
							break;
						case STATMOD_OR:
							w->plr.extrinsic[ ITEMDESC(*item).desc.armor.effects[j].stat ] |= ITEMDESC(*item).desc.armor.effects[j].amount;
							break;
					}
				}
		}
	}
	
	w->plr.extrinsic[STAT_DV]    += mod_from_attribute(STAT_DEXTERITY, 1, 2);
	w->plr.extrinsic[STAT_PV]    += mod_from_attribute(STAT_TOUGHNESS, 2, 3);
	w->plr.extrinsic[STAT_SPEED] += mod_from_attribute(STAT_DEXTERITY, 3, 2);
	
	w->plr.extrinsic[STAT_LIGHTRADIUS] *= w->plr.extrinsic[STAT_LIGHTRADIUS];
	w->plr.extrinsic[STAT_LIGHTRADIUS]++;
	
	w->plr.hps_max = w->plr.extrinsic[STAT_TOUGHNESS] + max(2, (5 + mod_from_attribute(STAT_TOUGHNESS, 0, 2))) * (w->plr.level - 1);
	w->plr.pps_max = w->plr.extrinsic[STAT_MANA] + max(1, (4 + mod_from_attribute(STAT_MANA, 0, 2))) * (w->plr.level - 1);
}
//}}}
//{{{
void enter_map(void)
{
	UI_TF_PlaceCamera(&w->tilefield, w->plr.x, w->plr.y);
	calc_light();
}
//}}}
//{{{
void follow_connection(void)
{
	spec_descriptor *connection;
	tile *t = &w->t[w->plr.y][w->plr.x];
	int target_level, source_level;
	
	if( t->type!=TILE_STAIRUP && t->type!=TILE_STAIRDN ) {
		message("You can't go up or down from here.");
		return;
	}
	
	connection = &((spec_descriptor*)CONSTFILEDEREF(w->mapdescs[w->level].special))[t->special];
	target_level = connection->destmap;
	
	if( target_level==0 )
	{
		if(prompt("Leave the dungeon"))
			die(0);
		else
			return;
	}
	
	w->t[w->plr.y][w->plr.x].flags &= ~TFLAG_OCCUPIED;
	
	save_level();
	source_level = w->level;
	w->maxlevel = max(w->maxlevel, w->level);
	w->level = target_level;
	if( !load_level(source_level, connection->destcoord) )
		gen_map(source_level, connection->destcoord );
	
	enter_map();
	draw_all_tiles();
	UI_TF_FullRedraw(&w->tilefield);
	timepass(1);
}
//}}}
//{{{
void plr_move(void)
{
	int move;
	
	if(w->plr.paraltime)
	{
		timepass(1);
		return;
	}
	
	move = READCHAR;
	
	switch(move) {
		
		/////////////////////////// MOVEMENT //////////////////////////////////
		
		case KEY_SOUTHWEST: plr_walk(-1,  1);  break;
		case KEY_SOUTH:     plr_walk( 0,  1);  break;
		case KEY_SOUTHEAST: plr_walk( 1,  1);  break;
		case KEY_WEST:      plr_walk(-1,  0);  break;
		case KEY_EAST:      plr_walk( 1,  0);  break;
		case KEY_NORTHWEST: plr_walk(-1, -1);  break;
		case KEY_NORTH:     plr_walk( 0, -1);  break;
		case KEY_NORTHEAST: plr_walk( 1, -1);  break;
		
		case KEY_WAIT:
			timepass(1);
			break;
		
		case KEY_STAIR:	// Use stair
			follow_connection();
			break;
		
		/////////////////////////// INVENTORY /////////////////////////////////

		case KEY_INVENTORY:	// Show inventory
			if(w->plr.numitems==0)
				message("You are empty-handed.");
			else
				pick_item(filter_none);
			break;
		
		case KEY_WEAR:	// Wear an item
			wear_item();
			break;
			
		case KEY_TAKEOFF:	// Take off
			takeoff_item();
			break;
			
		case KEY_DROP:	// Drop an item
			drop_item();
			break;
				
		case KEY_PICKUP:	// Pick up an item
			pickup();
			break;
		
		case KEY_CAST:
			cast_spell();
			break;
			
		case KEY_USE:	// Use an item
			use_item();
			break;
		
		case KEY_THROW:
			throw_item_from_menu();
			break;
			
		case KEY_FIRE:
			throw_readied_item();
			break;
		///////////////////////////// MISC ////////////////////////////////////
		
		case KEY_DISCOVERIES:
			list_discoveries();
			break;
		
		case KEY_SEARCH:	// Search
			plr_search();
			break;
		
		case KEY_ESC:	// ESC - quit
			if( prompt("Really quit") )
				die(0);
			break;
		
		case KEY_SAVEGAME:	// On
			save_game();
			quit();
			break;
		
		case KEY_F1:	// F1 - Help
			help();
			break;
		case KEY_F2:	// F2 - Character info
			char_info();
			break;
		case KEY_F3:	// F3 - About
			message("CalcRogue by Jim Babcock, 2002. This program is postcardware. If you enjoy it, send the author a postcard at 16 Sweeney Ridge Rd, Bedford, MA, USA, ZIP code 01730");
			break;
		
		case KEY_CLOSEDOOR:	// Close door
			manip_door(0);
			break;
		case KEY_OPENDOOR:	// Open door
			manip_door(1);
			break;
		
#		ifdef KEY_SCROLLING
			case KEY_SCROLL_RIGHT:
				UI_TF_Scroll(&w->tilefield, 8, 0);
				break;
			case KEY_SCROLL_DOWN:
				UI_TF_Scroll(&w->tilefield, 0, 8);
				break;
			case KEY_SCROLL_UP:
				UI_TF_Scroll(&w->tilefield, 0, -8);
				break;
			case KEY_SCROLL_LEFT:
				UI_TF_Scroll(&w->tilefield, -8, 0);
				break;
#		endif
		
		////////////////////////// UNIMPLEMENTED //////////////////////////////
		
		case KEY_F4:	// F4
			break;
		
		case KEY_KICK:	// Kick
		case KEY_RUN:		// Backspace - run
			break;
		
		///////////////////////////// DEBUG ///////////////////////////////////
#ifdef ALLOWDEBUG
		case KEY_F5:	// F5 - Debug menu
			debug();
			break;
#endif
	
		default:	// Debug: message the keycode of any unrecognized key pressed
			message("Unrecognized key code: %i", move);
			break;
	}
	return;
}
//}}}
//{{{
void place_player_randomly(void)
{
	int x, y;
	
retry:
	x = RANGE(MAPSIZE_X-1, 0);
	y = RANGE(MAPSIZE_Y-1, 0);
	if( !(w->tiledescs[w->t[y][x].type].passable) )
		goto retry;
	if( w->t[y][x].flags & TFLAG_OCCUPIED )
		goto retry;
	
	plr_moveto(x, y);
}
//}}}
//{{{
void plr_walk(ushort x, ushort y)
{
	int d;
	if( w->plr.conftime ) {
		message("You are confused.");
		d = RANGE(7,0);
		x = dir[d][0];
		y = dir[d][1];
	}
	x += w->plr.x;
	y += w->plr.y;
	
	if( w->t[y][x].flags & TFLAG_OCCUPIED && (w->plr.x != x || w->plr.y != y) )
	{
		plr_attack( monstbytile(x, y) );
		timepass(1);
		return;
	}
	
	if( w->tiledescs[w->t[y][x].type].passable )
		plr_moveto(x, y);
	else if(w->t[y][x].type==TILE_CDOOR) {
		w->t[y][x].type = TILE_ODOOR;
		draw_tile(x, y);
		calc_light();
	}
	
	timepass(1);
}
//}}}
//{{{
void plr_attack( int monstid )
{
	int damage;
	
	if( plr_attack_monster_roll(monstid, 0) )
	{
		damage = nrandom(w->plr.extrinsic[STAT_DAMAGE_MAX], w->plr.extrinsic[STAT_DAMAGE_MIN]);
		message("You hit the %s.", monstname(monstid));
		
		monst_takedamage(monstid, damage, 1);
	} else {
		message("You miss it.");
	}
}
//}}}
//{{{
void plr_takedamage(int amt)
{
	amt = max( amt - w->plr.extrinsic[STAT_PV], max(amt/4, 1) );
	if(amt==0) return;
	w->plr.hps -= amt;
	
	// If no hit points, die.
	if(w->plr.hps <= 0)	// Game over.
	{
		die(1);
	}
}
//}}}
//{{{
void plr_heal(int amt)
{
	w->plr.hps += amt;
	if(w->plr.hps > w->plr.hps_max)
		w->plr.hps = w->plr.hps_max;
}
//}}}
//{{{
const char* statmsg[2][5] = {
	{
		"You feel weaker.",
		"You feel clumsy.",
		"You feel out of shape.",
		"You feel tired."
		"You feel naive.",
	},
	{
		"You feel stronger.",
		"Your coordination improves.",
		"You feel tougher.",
		"You feel powerful.",
		"You feel skeptical.",
	}
  };
//}}}
//{{{
void plr_mod_stat(int which, int amt)
{
	message(statmsg[amt>0][which]);
	
	if( amt <= 0 ) {
		if(w->plr.intrinsic[which] > amt)
			w->plr.intrinsic[which] += amt;
		else
			w->plr.intrinsic[which] = 1;
	} else {
		w->plr.intrinsic[which] += amt;
	}
	
	update_player();
}
//}}}
//{{{
void satiate(int amt)
{
	w->plr.satiation += amt;
}
//}}}
//{{{
void plr_moveto(ushort x, ushort y)
{
	w->t[w->plr.y][w->plr.x].flags &= ~TFLAG_OCCUPIED;
	draw_tile(w->plr.x, w->plr.y);	// Redraw the tile the player stepped off of
	w->plr.x = x;
	w->plr.y = y;
	w->t[w->plr.y][w->plr.x].flags |= TFLAG_OCCUPIED | TFLAG_LIT;
	UI_TF_MoveCamera(&w->tilefield, w->plr.x, w->plr.y);
	draw_tile(w->plr.x, w->plr.y);	// Redraw the tile the player stepped onto
	calc_light();
	
	if(w->t[y][x].type == TILE_DARTTRAP) {
		w->t[y][x].flags &= ~TFLAG_HIDETRAP;
		message("You are hit by a dart!");
		plr_takedamage(RANGE(10,5));
	}

	if(w->t[y][x].numitems)
		pickup();
}
//}}}
//{{{
void manip_door(int open)
{
	int x, y;
	direction d = prompt_dir("In what direction");
	int success=0;
	
	x = w->plr.x + d.x;
	y = w->plr.y + d.y;
	
	if( open ) {
		if( w->t[y][x].type == TILE_CDOOR ) {
			w->t[y][x].type = TILE_ODOOR;
			success=1;
		}
	} else {
		if( w->t[y][x].type == TILE_ODOOR ) {
			w->t[y][x].type = TILE_CDOOR;
			success=1;
		}
	}
	calc_light();
	
	if( !success )
		message("You see no door there.");
	else
	{
		draw_tile(x, y);
		timepass(1);
	}
}
//}}}
//{{{
void plr_search(void)
{
	int x, y;
	
	timepass(1);
	
	for(y=w->plr.y-1; y<=w->plr.y+1; y++)
		for(x=w->plr.x-1; x<=w->plr.x+1; x++)
		{
			if(w->t[y][x].type == TILE_SDOOR && RANGE(2,0)==0)	// Found a secret door
			{
				w->t[y][x].type = TILE_CDOOR;
				message("You found a secret door!");
				draw_tile(x, y);
			}
			if(w->t[y][x].flags & TFLAG_HIDETRAP && RANGE(1,0)==0)	// Found a trap
			{
				w->t[y][x].flags &= ~TFLAG_HIDETRAP;
				message("You found a trap!");
				draw_tile(x, y);
			}
		}
}
//}}}
//{{{
void awardXP(int amt)
{
	int i;
	
	ulong xpneeded=50;
	
	w->plr.xp += amt;
	
	// If sufficient experience, gain a level.
	for(i=w->plr.level; i>0; i--)
		xpneeded += 25*i;
	
	if(w->plr.xp > xpneeded) {
		message("You gained a level!");
		w->plr.level++;
		w->plr.xp = 0;
		update_player();
	}
	
	w->plr.score += amt;
}
//}}}
const unsigned char playerScreenTabStops[3] = { 0, 50, 60 };
//{{{
void char_info(void)
{
	char outbuf[2048];
	Black_ClrScr();
	sprintf(
		outbuf,
		"Level %i %s\n"
		"\n"
		"Strength\t"    "%i\t" "(%i)\n"
		"Dexterity\t"   "%i\t" "(%i)\n"
		"Toughness\t"   "%i\t" "(%i)\n"
		"Mana\t"   	    "%i\t" "(%i)\n"
		"Willpower\t"   "%i\t" "(%i)\n"
		"\n"
		"Speed %i\n"
		"DV %i(%i)\n"
		"PV %i(%i)\n"
		"Damage %i-%i\n",
		w->plr.level,                (const char*)CONSTFILEDEREF(w->playerclasses[w->plr.class].name),
		w->plr.intrinsic[STAT_STRENGTH],   w->plr.extrinsic[STAT_STRENGTH],
		w->plr.intrinsic[STAT_DEXTERITY],  w->plr.extrinsic[STAT_DEXTERITY],
		w->plr.intrinsic[STAT_TOUGHNESS],  w->plr.extrinsic[STAT_TOUGHNESS],
		w->plr.intrinsic[STAT_MANA],       w->plr.extrinsic[STAT_MANA],
		w->plr.intrinsic[STAT_WILLPOWER],  w->plr.extrinsic[STAT_WILLPOWER],
		w->plr.extrinsic[STAT_SPEED],
		w->plr.extrinsic[STAT_DV],         w->plr.intrinsic[STAT_DV],
		w->plr.extrinsic[STAT_PV],         w->plr.intrinsic[STAT_PV],
		w->plr.extrinsic[STAT_DAMAGE_MIN], w->plr.extrinsic[STAT_DAMAGE_MAX]
	);
	setTabStops(3, playerScreenTabStops);
	draw_string(outbuf, NULL, 0, 0, SCREEN_COLUMNS, SCREEN_ROWS, 1);
	READCHAR;
	UI_TF_FullRedraw(&w->tilefield);
}

