#include "crogue.h"

#ifdef WIN32
int scratch;
#endif

#ifdef IS_CALCULATOR
const SCR_RECT fullScreen = (SCR_RECT){{0, 0, 159, 99}};
#endif

#ifdef REALCOMPUTER
#	ifndef min
int min(int a, int b)
{
	if(a<b)
		return a;
	else
		return b;
}
#	endif
#	ifndef max
int max(int a, int b)
{
	if(a<b)
		return b;
	else
		return a;
}
#	endif
#endif

//{{{
void catprintf(char *target, const char *format, ...)
{
	va_list varargs;
	
	while(*target) target++;
	
	va_start(varargs, format);
	vsprintf(target, format, varargs);
	va_end(varargs);
}
//}}}

//{{{
ushort tracevision(sshort x1, sshort y1, sshort x2, sshort y2)
{
	sshort xdist, ydist;
	sshort xi, yi;
	sshort xdir, ydir;
	
	xdir=(x1<x2)?1 : (x1==x2)?0:-1;
	ydir=(y1<y2)?1 : (y1==y2)?0:-1;
	
	xdist=abs(x1-x2); ydist=abs(y1-y2);
	xi=(ydist/2); yi=(xdist/2);
	
	// Hack: If tracing to a non-transparent tile (such as a wall), move one tile towards the player
	// This solves the corner and edge problems without introducing any noticeable false positives
	if( !w->tiledescs[w->t[y1][x1].type].transparent ) {
		x1 += xdir;
		y1 += ydir;
	}
	
	while(x1!=x2 || y1!=y2)
	{
		if( !w->tiledescs[w->t[y1][x1].type].transparent )
			return 0;
		
		xi+=xdist, yi+=ydist;
		if(xi>=ydist) {
			if( x1!=x2 )  x1 += xdir;
			xi -= ydist;
		}
		if(yi>=xdist) {
			if( y1!=y2 )  y1 += ydir;
			yi -= xdist;
		}
	}
	return 1;
}
//}}}
//{{{
coord trace_path(int x, int y, direction dir, int distance)
{
	while(distance)
	{
		x += dir.x;
		y += dir.y;
		if( w->t[y][x].flags & TFLAG_OCCUPIED || !w->tiledescs[w->t[y][x].type].passable )
			break;
		distance--;
	}
	return COORD(x, y);
}
//}}}
//{{{
void illuminate(int x, int y, int radiussquared)
{
	int i, j;
	
	for(i=0; i<MAPSIZE_Y; i++)
		for(j=0; j<MAPSIZE_X; j++)
		{
			if( (x-j)*(x-j) + (y-i)*(y-i) <= radiussquared && tracevision(j, i, x, y) )
				w->t[i][j].flags |= TFLAG_INTRINSIC_LIGHT;
		}
}
//}}}

//{{{
void Black_ClrScr(void)
{
#ifdef IS_CALCULATOR
	GraySetAMSPlane(LIGHT_PLANE);
	clrscr();
	GraySetAMSPlane(DARK_PLANE);
	clrscr();
#else
	clrscr();
#endif
}
//}}}
//{{{
void Black_DrawChar (short x, short y, char c, short Attr)
{
#ifdef IS_CALCULATOR
	GraySetAMSPlane(LIGHT_PLANE);
	DrawChar(x, y, c, Attr);
	GraySetAMSPlane(DARK_PLANE);
	DrawChar(x, y, c, Attr);
#else
	gotoxy(x, y);
	addch(c);
#endif
}
//}}}
//{{{
void Black_DrawStr(short x, short y, const char *str, short Attr)
{
#ifdef IS_CALCULATOR
	GraySetAMSPlane(LIGHT_PLANE);
	DrawStr(x, y, str, Attr);
	GraySetAMSPlane(DARK_PLANE);
	DrawStr(x, y, str, Attr);
#else
	gotoxy(x, y);
	printw((char*)str);
#endif
}
//}}}
#ifdef IS_CALCULATOR
//{{{
void Black_ScrRectFill(const SCR_RECT *rect, const SCR_RECT *clip, short Attr)
{
	GraySetAMSPlane(LIGHT_PLANE);
	ScrRectFill(rect, clip, Attr);
	GraySetAMSPlane(DARK_PLANE);
	ScrRectFill(rect, clip, Attr);
}
//}}}
//{{{
void Black_DrawLine(short x0, short y0, short x1, short y1, short Attr)
{
	GraySetAMSPlane(LIGHT_PLANE);
	DrawLine(x0, y0, x1, y1, Attr);
	GraySetAMSPlane(DARK_PLANE);
	DrawLine(x0, y0, x1, y1, Attr);
}
//}}}
#endif

//{{{
// High-byte is maximum
// Low-byte is minimum
uchar xrandom(uint range)
{
	uint min, max;
	min =  range & 0x00FF;
	max = (range & 0xFF00) >> 8;
	return rand()%(max-min+1) + min;
}
//}}}
//{{{
uchar rrandom(range r)
{
	return rand()%(r.max-r.min+1) + r.min;
}
//}}}
//{{{
sint nrandom(uint max, uint min)
{
	return rand()%(max-min+1) + min;
}
//}}}
//{{{
// Pluralize according this rule:
//   Strings of the form "x of y" become "(x)s of y"
//   Other strings have "s" appended
char *pluralize(const char *str)
{
	int i=0, ii=0, xofyform=0;
	static char ret[64];
	
	while(str[i] != '\0') {
		if( str[i]==' ' && str[i+1]=='o' && str[i+2]=='f' ) {
			ret[ii++]='s';
			xofyform = 1;
		}
		ret[ii] = str[i];
		i++;
		ii++;
	}
	if( !xofyform )
		ret[ii++]='s';
	ret[ii]='\0';
	
	return ret;
}
//}}}
//{{{
monstdesc *find_monster_description(int m)
{
	return (monstdesc*)deref_file_ptr(w->m[m].type);
}
//}}}

//{{{
void init_shuffle_tab(void)
{
	int i, j, k, pos;
	uint temp;
	uint *sizes;
	
	w->shuffletranslation = (uint*)malloc( sizeof(uint) * w->desc.shuffletabsize );
	if(!w->shuffletranslation)
		fatal("Out of memory.");
	
	// Make an identity table
	for(i=0; i<w->desc.shuffletabsize; i++)
		w->shuffletranslation[i] = i;
	
	// Shuffle things around
	sizes = (uint*)CONSTFILEDEREF(w->desc.stsizes);
	pos = 0;
	for(i=0; i<w->desc.numshuffletabs; i++)
	{
		for(j=0; j<sizes[i]; j++)
		{
			k = pos+RANGE(sizes[i]-1, 0);
			
			temp = w->shuffletranslation[k];
			w->shuffletranslation[k] = w->shuffletranslation[pos+j];
			w->shuffletranslation[pos+j] = temp;
		}
		pos += sizes[i];
	}
}
//}}}

//{{{
const filelink filelink_null = { 0, 0, 0 };

enum {
	jt_unconditional=0,
	jt_playerlevel,
	jt_dungeonlevel,
	jt_avglevel,
	jt_class
};
//}}}
//{{{
typedef struct jumptabhead
{
	uint numEntries;
	uint condition;
} jumptabhead;
//}}}
//{{{
typedef struct jumptabentry // 8B (!!!)
{
	filelink target;
	uint freq;
	range condition;
} jumptabentry;
//}}}
//{{{
const void *deref_file_ptr(filelink p)
{
	return (char*)(deref_file_ptr_partial(p).offset + (char*)w->constfileoffset);
}
//}}}
//{{{
filelink deref_file_ptr_partial(filelink p)
{
	char *target;
	
	while(p.flags & (PTRFLAG_JUMPTABLE|PTRFLAG_SHUFFLETAB))
	{
		target = (char*)((char*)w->constfileoffset + p.offset);
		
		if(p.flags & PTRFLAG_JUMPTABLE)
			p = deref_jump_table(target);
		else if(p.flags & PTRFLAG_SHUFFLETAB)
			p = w->shuffledata[w->shuffletranslation[p.offset]];
	}
	
	return p;
}
//}}}
//{{{
filelink deref_jump_table(const char *table)
{
	const jumptabhead *header = (jumptabhead*)table;
	const jumptabentry *body = (jumptabentry*)(header+1);
	uint low, high, mid, target, conditionresult;
	
	while(1)
	{
		// Weighted-pick an entry (binary search)
		low = 1;
		high = header->numEntries-1;
		target = (rand() % body[high].freq)+1;
		
		while(1) {
			mid = (low+high)/2;
			
			if(low>=high)
				break;

			if(body[mid].freq >= target) {
				if(body[mid-1].freq < target)
					break;
				else
					high = mid-1;
			} else {
				low = mid+1;
			}
		}
		
		// Test condition
		switch(header->condition)
		{
			default:
			case jt_unconditional:
				conditionresult = 1;
				break;
			case jt_playerlevel:
				conditionresult = w->plr.level;
				break;
			case jt_dungeonlevel:
				conditionresult = w->mapdescs[w->level].difficulty;
				break;
			case jt_avglevel:
				conditionresult = (w->mapdescs[w->level].difficulty + w->plr.level)/2;
				break;
			case jt_class:
				conditionresult = w->plr.class;
				break;
		}
		if(conditionresult >= body[mid].condition.min && conditionresult <= body[mid].condition.max)
			break;
	}
	return body[mid].target;
}
//}}}

